@@ -6,6 +6,7 @@ from typing import Any
 from typing import Iterator
 
 from tomlkit._compat import decode
+from tomlkit._types import _CustomDict
 from tomlkit._utils import merge_dicts
 from tomlkit.exceptions import KeyAlreadyPresent
 from tomlkit.exceptions import NonExistentKey
@@ -19,7 +20,6 @@ from tomlkit.items import SingleKey
 from tomlkit.items import Table
 from tomlkit.items import Trivia
 from tomlkit.items import Whitespace
-from tomlkit.items import _CustomDict
 from tomlkit.items import item as _item
 
 
@@ -44,6 +44,7 @@ class Container(_CustomDict):
         return self._body
 
     def unwrap(self) -> dict[str, Any]:
+        """Returns as pure python object (ppo)"""
         unwrapped = {}
         for k, v in self.items():
             if k is None:
@@ -64,6 +65,7 @@ class Container(_CustomDict):
 
     @property
     def value(self) -> dict[str, Any]:
+        """The wrapped dict value"""
         d = {}
         for k, v in self._body:
             if k is None:
